/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
 * http://www.gnu.org/software/gnugo/ for more information.      *
 *                                                               *
 * Copyright 1999 and 2000 by the Free Software Foundation.      *
 *                                                               *
 * This program is free software; you can redistribute it and/or *
 * modify it under the terms of the GNU General Public License   *
 * as published by the Free Software Foundation - version 2.     *
 *                                                               *
 * This program is distributed in the hope that it will be       *
 * useful, but WITHOUT ANY WARRANTY; without even the implied    *
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
 * PURPOSE.  See the GNU General Public License in file COPYING  *
 * for more details.                                             *
 *                                                               *
 * You should have received a copy of the GNU General Public     *
 * License along with this program; if not, write to the Free    *
 * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
 * Boston, MA 02111, USA                                         *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include "interface.h"

#include "random.h"
#include "sgfana.h"

void
play_solo(int boardsize, int moves)
{
  Gameinfo  gameinfo;

  int passes = 0; /* num. consecutive passes */
  int to_move = BLACK;
  int move_val;
  int mycolor;
  int yourcolor;
  double t1, t2;
  int benchmark = moves;
  
  /* It tends not to be very imaginative in the opening,
   * so we scatter a few stones randomly to start with.
   * We add two random numbers to reduce the probability
   * of playing stones near the edge.
   */
  
  int n = 6 + 2*gg_rand()%5;
  int i, j;

  int handicap;
  int seed;
  float komi;
  
  ginfo_clear(&gameinfo, boardsize);

  komi     = gameinfo.komi;
  seed     = gameinfo.seed;
  handicap = gameinfo.handicap;
  mycolor  = gameinfo.computer_player;
  
  yourcolor = OTHER_COLOR(mycolor);
  
  sgffile_write_gameinfo(&gameinfo, "solo");
  
  /* Generate some random moves. */
  if (boardsize > 6) {
    do {
      do {
	i = (gg_rand() % 4) + (gg_rand() % (boardsize - 4));
	j = (gg_rand() % 4) + (gg_rand() % (boardsize - 4));
      } while (gameinfo.position.board[i][j] != EMPTY);
      
      position_play_move(&gameinfo.position, i, j, to_move);
      sgffile_move_made(i, j, to_move, 0);
      sgfAddPlay(0, to_move, i, j);
      sgfAnalyzer(0);
      to_move = OTHER_COLOR(to_move);
    } while (--n > 0);
  }
  
  t1 = gg_gettimeofday();
  
  while (passes < 2 && --moves >= 0 && !time_to_die) {
    move_val = position_genmove(&gameinfo.position, &i, &j, to_move,
				gameinfo.move_number);
    position_play_move(&gameinfo.position, i, j, to_move);
    if (move_val < 0) {
      ++passes;
      printf("%s(%d): Pass\n", to_move==BLACK ? "Black" : "White",
	     gameinfo.move_number);
    } else {
      passes = 0;
      gameinfo.move_number++;
      gprintf("%s(%d): %m\n", to_move==BLACK ? "Black" : "White",
	      gameinfo.move_number, i, j);
    }
    sgffile_move_made(i, j, to_move, move_val);
    sgfAddPlay(0, to_move, i, j);
    sgfAnalyzer(move_val);
    to_move=OTHER_COLOR(to_move);
  }
  t2 = gg_gettimeofday();
  
  /* Two passes and it's over. */
  who_wins(mycolor, komi, stdout);
  if (t2 == t1) {
    printf("%.3f moves played\n", (double) (benchmark-moves));
  } else {
    printf("%.3f moves/sec\n", (benchmark-moves)/(t2-t1));
  }
}


/*
 * Local Variables:
 * tab-width: 8
 * c-basic-offset: 2
 * End:
 */
